using System;
using System.Xml;
using System.Xml.XPath;
using System.Runtime.Remoting;
using System.Runtime.CompilerServices;
using Interfaces;
using System.Collections;
using Helpers;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// represents a Tuple
	/// </summary>
	[Serializable]
	public class XMLTuple : MarshalByRefObject, ITuple
	{
		/// <summary>
		/// the tuple's value
		/// </summary>
		private Object wert;
		/// <summary>
		/// Extended version of XmlDocument
		/// </summary>
		protected ExtendedXmlDocument xmldoc;
		/// <summary>
		/// counter of tuples (inside of tuple)
		/// </summary>
		private int tuplecount = 1;	// das document wird bereits mit 1 initialisiert
		/// <summary>
		/// counter of fields (inside of tuple)
		/// </summary>
		private int fieldcount = 1;
		/// <summary>
		/// reference to create new fields
		/// </summary>
		protected XMLField field;
		
		/// <summary>
		/// On construction the class creates an XmlDocument for further use
		/// </summary>
		public XMLTuple()
		{
			xmldoc = XMLTupleHelper.createRootDocument();
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing an int
		/// </summary>
		/// <param name="i">the value of the XMLField</param>
		public XMLTuple(int i)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(i);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing a byte
		/// </summary>
		/// <param name="b">the value of the XMLField</param>
		public XMLTuple(byte b)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(b);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing a SByte
		/// </summary>
		/// <param name="sb">the value of the XMLField</param>
		public XMLTuple(SByte sb)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(sb);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing a short
		/// </summary>
		/// <param name="s">the value of the XMLField</param>
		public XMLTuple(short s)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(s);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing a long
		/// </summary>
		/// <param name="l">the value of the XMLField</param>
		public XMLTuple(long l)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(l);
			add(field);
		}
		/// <summary>
		/// constructs a XMLTuple with a XMLField containing a float
		/// </summary>
		/// <param name="f">the value of the XMLField</param>
		public XMLTuple(float f)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(f);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing a double
		/// </summary>
		/// <param name="d">the value of the XMLField</param>
		public XMLTuple(double d)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(d);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing a char
		/// </summary>
		/// <param name="c">the value of the XMLField</param>
		public XMLTuple(char c)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(c);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing a string
		/// </summary>
		/// <param name="s">the value of the XMLField</param>
		public XMLTuple(String s)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			field = new XMLField(s);
			add(field);
		}
		/// <summary>
		/// constructs an XMLTuple with an XMLField containing an object
		/// </summary>
		/// <param name="o">the value of the XMLField</param>
		public XMLTuple(Object o)
		{
			try 
			{
				wert = o;
				xmldoc = XMLTupleHelper.createRootDocument();
				add(wert);
			} 
			catch(Exception e) 
			{
				Console.WriteLine("Exception on creation of XMLTuple : " + e.Message); 
			}
		}
		/// <summary>
		/// constructs an XMLTuple with an ITuple
		/// </summary>
		/// <param name="t">the ITuple </param>
		public XMLTuple(ITuple t)
		{			
			xmldoc = XMLTupleHelper.createRootDocument();
			add(t);				
		}
		/// <summary>
		/// Stores an IField in the tuple
		/// </summary>
		/// <param name="f">the IField</param>
		public XMLTuple(IField f)
		{
			xmldoc = XMLTupleHelper.createRootDocument();
			add(f);
		}
		/// <summary>
		/// returns the XmlDokument contains the Tuple
		/// </summary>
		/// <returns>
		/// the XmlDokument which contains the Tuple
		/// </returns>
		public ExtendedXmlDocument getXmlDocument() 
		{
			return xmldoc;
		}
		/// <summary>
		/// Adds an ITuple to current tuple
		/// </summary>
		/// <param name="t">
		/// the ITuple to add
		/// </param>
		public void add(ITuple t)
		{
			wert=t;
			XmlNode newNode = XMLTupleHelper.convertToXML(xmldoc, t);
			xmldoc.DocumentElement.AppendChild(newNode);
			increasecount();
		}
		/// <summary>
		/// Adds an object to current tuple
		/// </summary>
		/// <param name="obj">
		/// the object to add
		/// </param>
		public void add(Object t) 		
		{
			XmlNode newNode;
			newNode = XMLTupleHelper.buildSoapNode(xmldoc, t);			
			xmldoc.DocumentElement.AppendChild(newNode);
			incFieldcount();
		}
		/// <summary>
		/// Adds an XMLField to current tuple
		/// </summary>
		/// <param name="f">
		/// the IField to add
		/// </param>
		public void add(IField f)
		{
			XmlNode newNode;
			if(f.FieldValue is ExtendedXmlDocument)
			{
				newNode  = XMLTupleHelper.convertFieldToXML(xmldoc, (ExtendedXmlDocument) f.FieldValue);
			}
			else if(f.FieldValue is XmlNode) 
			{
				newNode = xmldoc.ImportNode(f.FieldValue as XmlNode, true);
			} 
			
			else 
			{
				newNode = XMLTupleHelper.convertFieldToXML(xmldoc,(XMLField) f);				
			}
			xmldoc.DocumentElement.AppendChild(newNode);	
			incFieldcount();
		}
		
		/// <summary>
		/// imports an xmlstring representing a tuple.
		/// </summary>
		/// <param name="xmlstring"></param>
		public void importXMLTuple(XmlNode node) 
		{	
			ExtendedXmlDocument newdoc = new ExtendedXmlDocument();
			XmlNode n = newdoc.ImportNode(node, true);
			newdoc.DocumentElement.AppendChild(n);	
		}
		/// <summary>
		/// returns an enumerator of all elements that are contained
		/// </summary>
		/// <returns>
		/// returns an enumerator of all elements that are contained
		/// </returns>
		public IEnumerator getList() 
		{
			return xmldoc.GetEnumerator();
		}
		/// <summary>
		/// gets the TupleID
		/// </summary>
		/// <returns>
		/// returns the TupleID
		/// </returns>
		public int getTupleID() 
		{
			return 0;
		}
		/// <summary>
		/// writes the XmlDokument into a file
		/// </summary>
		/// <param name="name">
		/// the filename
		/// </param>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public void writeToFile(String name) 
		{
			XmlTextWriter xmltw = new XmlTextWriter(name, System.Text.Encoding.UTF8);
			xmldoc.Save(xmltw);
			xmltw.Flush();
			xmltw.Close();
		}
		/// <summary>
		/// the string representation of Tuple
		/// </summary>
		/// <returns>
		/// returns the string representation of Tuple
		/// </returns>
		public override string ToString() 
		{
			XMLPrinter mpr = new XMLPrinter();
			return mpr.printXML(xmldoc.ChildNodes);
		}
		/// <summary>
		/// increments the number of XMLFields in the current XMLTuple
		/// </summary>
		public void incFieldcount()
		{
			((XmlElement) xmldoc.DocumentElement).SetAttribute("fieldcount", fieldcount.ToString());
			fieldcount++;
		}

		/// <summary>
		/// increments the  the number of Tuples in the current XMLTuple
		/// </summary>
		public void increasecount() 
		{
			//increase count in xml and counter
			((XmlElement) xmldoc.DocumentElement).SetAttribute("tuplecount", tuplecount.ToString());
			tuplecount++;
		}
		/// <summary>
		/// gets the number of Tuples
		/// </summary>
		/// <returns>
		/// returns the number of Tuples
		/// </returns>
		public int TupleCount
		{	
			get{ return(Int32.Parse((this.getXmlDocument().DocumentElement).GetAttribute("tuplecount")));}
		}
		/// <summary>
		/// gets the number of Fields
		/// </summary>
		/// <returns>
		/// returns the number of Fields
		/// </returns>
		public int FieldCount
		{	
			get{return (Int32.Parse((this.getXmlDocument().DocumentElement).GetAttribute("fieldcount")));}
		}
		/// <summary>
		/// gets the fields of the first level
		/// </summary>
		/// <returns>an ArrayList containing the fields</returns>
		public ArrayList getFields()
		{
			ArrayList fieldlist = new ArrayList();
			XMLField field;
			XmlNodeList list = this.getXmlDocument().DocumentElement.ChildNodes;
			if(list.Count == 0) Console.WriteLine("list ist leer");
			foreach(XmlNode n in list)
			{
				
				if(n.Name.Equals("Field"))
				{	
					Console.WriteLine("Feld gefunden: "+n.Name);
					string type = ((XmlElement)n).GetAttribute("type");
					field = new XMLField(n.InnerXml);
					field.Type=type;
					fieldlist.Add(field);	
				}
			}
			if(fieldlist.Count == 0) Console.WriteLine("arraylist ist leer");
			return fieldlist;
		}
		/// <summary>
		/// gets the tuples of the first level
		/// </summary>
		/// <returns>an ArrayList containing the tuples</returns>
		public ArrayList getTuples()
		{
			ArrayList tuplelist = new ArrayList();
			XmlNodeList list = ((XMLTuple)this).getXmlDocument().DocumentElement.ChildNodes;
			XMLTuple t ;
			foreach(XmlNode n in list)
			{
				if(n.Name.Equals("Tuple"))
				{
					t = new XMLTuple();
					t.getXmlDocument().LoadXml(n.OuterXml);
					tuplelist.Add(t);
				}
			}
			return tuplelist;
		}
		/// <summary>
		/// gets the entries (tuples and fields) of the first level
		/// </summary>
		/// <returns>an ArrayList containing the sequence of tuples and fields</returns>
		public ArrayList getEntries()
		{
			ArrayList entries = new ArrayList();
			XmlNodeList list = ((XMLTuple)this).getXmlDocument().DocumentElement.ChildNodes;
			XMLTuple t ;
			XMLField field;
			foreach(XmlNode n in list)
			{
				if(n.Name.Equals("Tuple"))
				{
					t = new XMLTuple();
					t.getXmlDocument().LoadXml(n.OuterXml);
					entries.Add(t);
				}else if(n.Name.Equals("Field"))
				{	
					Console.WriteLine("Feld gefunden: "+n.Name);
					string type = ((XmlElement)n).GetAttribute("type");
					field = new XMLField(n.InnerXml);
					field.Type=type;
					entries.Add(field);	
				}
			}
			return entries;
		
		}
		#region
		/*/// <summary>
		/// 
		/// </summary>
		/// <param name="t">
		/// the Tuple which searched
		/// </param>
		/// <returns>
		/// true if Tuple contains ITuple, false otherwise
		/// </returns>
		public bool XMLTupleExists(ITuple t) 
		{
			if (true) return true;
			else return false;
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="f">
		/// the Field which searched
		/// </param>
		/// <returns>
		/// true if Tuple contains the Field, false otherwise
		/// </returns>
		public bool XMLFieldExists(IField f)
		{
			return true;
		}
		/// <summary>
		/// gets the tuples as a XmlNodeList
		/// </summary>
		/// <returns>
		/// returns the XmlNodeList containing the Tuples
		/// </returns>
		public XmlNodeList getXMLTuples() 
		{
			return (xmldoc.ChildNodes);
		}
		/// <summary>
		/// gets the Field with index i
		/// </summary>
		/// <param name="i">
		/// the index of the Field
		/// </param>
		/// <returns>
		/// returns the Field with index i
		/// </returns>
		public IField getField (int i) 
		{
			return new XMLField();
		}*/
		#endregion
	}
}